#pragma once

#include <QString>
#include <QList>
#include <QHash>
#include "ast_basic.h"
#include "libsyntax_global.h"

namespace ast_gen
{
	class TokenAccess;

	/**
	 * @brief 
	 */
	class SyntaxElement {
	public:
		/**
		 * @brief ͱ
		 * @return
		 */
		virtual int typeMark() const = 0;

		/**
		 * Ƿڵ.
		 *
		 * \return ־
		 */
		virtual bool isAnonymous() const = 0;

		/**
		 * @brief ļ·
		 * @return
		 */
		virtual QString path() const = 0;

		/**
		 * @brief ԪǩԪطؿַ
		 * @return
		 */
		virtual QString signature() const = 0;

		/**
		 * @brief ȡԪ
		 * @return
		 */
		virtual std::shared_ptr<const SyntaxElement> parent() const = 0;

		/**
		 * @brief ԪToken
		 * @return
		 */
		virtual QList<std::shared_ptr<const TokenAccess>> selfTokens() const = 0;
	};

	/**
	 * @brief ﷨Ԫʽӿ
	 */
	class LIBSYNTAX_EXPORT ElementAccess {
	private:
		std::shared_ptr<const SyntaxElement> peers;
		QList<std::shared_ptr<const ElementAccess>> children_store;

	public:
		ElementAccess(std::shared_ptr<const SyntaxElement> point);

		std::shared_ptr<const SyntaxElement> element() const;

		QList<std::shared_ptr<const ElementAccess>> children() const;
		void appendChild(std::shared_ptr<const ElementAccess> inst);


		/**
		 * @brief ȡԪToken
		 * @return
		 */
		virtual QList<std::shared_ptr<const TokenAccess>> tokens() const;
	};

	class LIBSYNTAX_EXPORT TokenAccess {
	private:
		std::shared_ptr<const ast_gen::SyntaxElement> element_bind;
		std::shared_ptr<const lib_token::Token> token_store;

	public:
		TokenAccess(std::shared_ptr<const ast_gen::SyntaxElement> elm_inst, std::shared_ptr<const lib_token::Token> token_inst);
		virtual std::shared_ptr<const ast_gen::SyntaxElement> bind() const;
		virtual std::shared_ptr<const lib_token::Token> token() const;
	};

	/**
	 * @brief Ԫض
	 */
	class LIBSYNTAX_EXPORT GlobalElement : public SyntaxElement {
	private:
		QList<std::shared_ptr<const SyntaxElement>> children_store;
		QString names_store;
		QHash<QString, std::shared_ptr<const SyntaxElement>> node_cache;

	public:
		GlobalElement(const QString& name);

		virtual void clearCache();
		virtual void appendToCache(std::shared_ptr<const ast_gen::SyntaxElement> named_node);
		/**
		 * @brief ͨڵǩȡڵ
		 * @param signature ȫǩ
		 * @return
		 * @throws ûָڵ׳쳣
		 */
		virtual std::shared_ptr<const ast_gen::SyntaxElement> getNamedNodeBy(int type, const QString& signature) const;

		// ParseElement interface
	public:
		virtual int typeMark() const override;
		virtual bool isAnonymous() const override;
		virtual QString signature() const override;
		virtual QString path() const override;
		virtual std::shared_ptr<const SyntaxElement> parent() const override;
		virtual QList<std::shared_ptr<const TokenAccess>> selfTokens() const override;
	};

	/**
	 * @brief ﷨ڵ
	 */
	class ExprVisitor {
	public:
		/**
		 * @brief ʽڵ㣬ת﷨Ԫʵ
		 * @param expr ʽڵ
		 * @param pnode ڵ
		 * @return ýڵתɵ﷨Ԫؽڵ
		 */
		virtual std::shared_ptr<const SyntaxElement> visit(std::shared_ptr<const ast_basic::ExprNode> expr, std::shared_ptr<const SyntaxElement> pnode) const = 0;
	};

	/**
	 * @brief ﷨
	 */
	class LIBSYNTAX_EXPORT ExprTreeParser {
	private:
		std::shared_ptr<const ExprVisitor> visitor_store;
		void tidy_branch(std::shared_ptr<ElementAccess> root, std::shared_ptr<const SyntaxElement> node) const;

	public:
		ExprTreeParser(std::shared_ptr<const ExprVisitor> vistor);

		/**
		 * @brief ﷨ݣɵ﷨
		 * @param name ɲ
		 * @param forest ʽɭ֣ȫıʽ
		 * @return ﷨Ҷӽڵ
		 */
		QList<std::shared_ptr<const SyntaxElement>> parse(const QString& name, const QList<std::shared_ptr<const ast_basic::ExprNode>>& forest) const;

		/**
		 * @brief ﷨ڵ㣬﷨Ԫط
		 * @param nodes
		 * @return
		 */
		std::shared_ptr<const ElementAccess> tidy(const QList<std::shared_ptr<const SyntaxElement>>& nodes) const;
	};
}