#pragma once

#include "libtoken_global.h"
#include <QList>
#include <QString>
#include <memory>

namespace lib_token {
    class TokenDefine;

    /**
     * @brief Դʵ
     */
    class WordBase {
    public:
        /**
         * @brief Դļ·
         * @return
         */
        virtual QString file() const = 0;
        /**
         * @brief 
         * @return
         */
        virtual QString content() const = 0;
        /**
         * @brief Դк
         * @return
         */
        virtual int row() const = 0;
        /**
         * @brief Դк
         * @return
         */
        virtual int column() const = 0;
    };

    /**
     * @brief token
     */
    class Token : public WordBase {
    public:
        /**
         * @brief tokenƹ
         * @return
         */
        virtual std::shared_ptr<const TokenDefine> define() const = 0;
    };

    /**
     * @brief tokenƶ
     */
    class TokenDefine {
    public:
        /**
         * @brief 
         * @return
         */
        virtual QString name() const = 0;
        /**
         * @brief ƹؼ
         * @return
         */
        virtual QString regex() const = 0;

        /**
         * @brief ﴦ
         * @param content 
         * @return tuple<token/null,remains> 
         */
        virtual std::tuple<std::shared_ptr<const Token>, std::shared_ptr<const WordBase>> analysis(std::shared_ptr<const WordBase> content) const = 0;
    };

    /**
     * @brief 쳣
     */
    class LIBTOKEN_EXPORT TokenException {
    private:
        QString msg_store;

    public:
        TokenException(const QString& message);
        virtual QString message() const;
    };

    /**
     * @brief δԭʼ
     */
    class LIBTOKEN_EXPORT WordPeaks : public WordBase {
    private:
        int row_n, col_n;
        QString text_n, path_p;

    public:
        WordPeaks(int r, int c, const QString& t, const QString& p);

        // WordBase interface
    public:
        virtual QString file() const override;
        virtual QString content() const override;
        virtual int row() const override;
        virtual int column() const override;
    };

    /**
     * @brief ʷɹ
     */
    class LIBTOKEN_EXPORT TokenInst : public Token {
    private:
        int row_n, col_n;
        QString text_n, path_p;
        std::shared_ptr<const TokenDefine> type_def;

    public:
        TokenInst(int r, int c, const QString& t, const QString& p, std::shared_ptr<const TokenDefine> type);

        // WordBase interface
    public:
        virtual QString file() const override;
        virtual QString content() const override;
        virtual int row() const override;
        virtual int column() const override;

        // Token interface
    public:
        virtual std::shared_ptr<const TokenDefine> define() const override;
    };

    /**
     * @brief ʷȡ
     */
    class LIBTOKEN_EXPORT TokenReader {
    private:
        QList<std::shared_ptr<const TokenDefine>> rules_store;

        QList<std::shared_ptr<const Token>> parse_line(int row, const QString& line_text, const QString& path) const;
        std::tuple<std::shared_ptr<const Token>, std::shared_ptr<const WordBase>> parse_token(std::shared_ptr<const WordBase> word) const;

    public:
        TokenReader(const QList<std::shared_ptr<const TokenDefine>> rulers);

        QList<std::shared_ptr<const Token>> extractFrom(const QString& path) const;
    };

} // namespace lib_token